@@ -8,8 +8,10 @@
 
 #include "hbuf.h"
 #include "hstring.h"
+#include "LRUCache.h"
 
 #define HTTP_HEADER_MAX_LENGTH      1024        // 1K
+#define FILE_CACHE_MAX_NUM          100
 #define FILE_CACHE_MAX_SIZE         (1 << 22)   // 4M
 
 typedef struct file_cache_s {
@@ -36,6 +38,7 @@ typedef struct file_cache_s {
     }
 
     bool is_complete() {
+        if(S_ISDIR(st.st_mode)) return filebuf.len > 0;
         return filebuf.len == st.st_size;
     }
 
@@ -54,17 +57,13 @@ typedef struct file_cache_s {
 } file_cache_t;
 
 typedef std::shared_ptr<file_cache_t>           file_cache_ptr;
-// filepath => file_cache_ptr
-typedef std::map<std::string, file_cache_ptr>   FileCacheMap;
 
-class FileCache {
+class FileCache : public hv::LRUCache<std::string, file_cache_ptr> {
 public:
-    FileCacheMap    cached_files;
-    std::mutex      mutex_;
     int             stat_interval;
     int             expired_time;
 
-    FileCache();
+    FileCache(size_t capacity = FILE_CACHE_MAX_NUM);
 
     struct OpenParam {
         bool need_read;
@@ -82,8 +81,8 @@ public:
         }
     };
     file_cache_ptr Open(const char* filepath, OpenParam* param);
+    bool Exists(const char* filepath) const;
     bool Close(const char* filepath);
-    bool Close(const file_cache_ptr& fc);
     void RemoveExpiredFileCache();
 
 protected:
