@@ -251,13 +251,35 @@ int HttpHandler::invokeHttpHandler(const http_handler* handler) {
 
 void HttpHandler::onHeadersComplete() {
     // printf("onHeadersComplete\n");
-    int status_code = handleRequestHeaders();
-    if (status_code != HTTP_STATUS_OK) {
-        error = ERR_REQUEST;
-        return;
+    handleRequestHeaders();
+    if (service->headerHandler) {
+        const int status_code = customHttpHandler(service->headerHandler);
+        if (status_code != HTTP_STATUS_OK && status_code != HTTP_STATUS_NEXT) {
+            SetError(ERR_REQUEST, static_cast<http_status>(status_code));
+            return;
+        }
     }
 
     HttpRequest* pReq = req.get();
+    const char *p = pReq->path.c_str();
+    while (*p != '\0') {
+        switch (*p) {
+            case '%':
+                if (p[1] != '0') break;
+                if (p[2] != 'd' && p[2] != 'D' && p[2] != 'a' && p[2] != 'A') break;
+            case '\r':
+            case '\n':
+                // fix CVE-2023-26147
+                hloge("[%s:%d] Illegal crlf path: %s", ip, port, pReq->path.c_str());
+                SetError(ERR_REQUEST);
+                return;
+
+            default:
+                break;
+        }
+        ++p;
+    }
+
     if (service && service->pathHandlers.size() != 0) {
         service->GetRoute(pReq, &api_handler);
     }
@@ -339,7 +361,7 @@ void HttpHandler::onMessageComplete() {
     }
 }
 
-int HttpHandler::handleRequestHeaders() {
+void HttpHandler::handleRequestHeaders() {
     HttpRequest* pReq = req.get();
     pReq->scheme = ssl ? "https" : "http";
     pReq->client_addr.ip = ip;
@@ -367,16 +389,6 @@ int HttpHandler::handleRequestHeaders() {
 
     // printf("url=%s\n", pReq->url.c_str());
     pReq->ParseUrl();
-    // printf("path=%s\n",  pReq->path.c_str());
-    // fix CVE-2023-26147
-    if (pReq->path.find("%") != std::string::npos) {
-        std::string unescaped_path = HUrl::unescape(pReq->path);
-        if (unescaped_path.find("\r\n") != std::string::npos) {
-            hlogw("Illegal path: %s\n",  unescaped_path.c_str());
-            resp->status_code = HTTP_STATUS_BAD_REQUEST;
-            return resp->status_code;
-        }
-    }
 
     if (proxy) {
         // Proxy-Connection
@@ -404,7 +416,6 @@ int HttpHandler::handleRequestHeaders() {
     }
 
     // TODO: rewrite url
-    return HTTP_STATUS_OK;
 }
 
 void HttpHandler::handleExpect100() {
@@ -574,7 +585,7 @@ int HttpHandler::defaultStaticHandler() {
             if (service->largeFileHandler) {
                 status_code = customHttpHandler(service->largeFileHandler);
             } else {
-                status_code = defaultLargeFileHandler();
+                status_code = defaultLargeFileHandler(filepath);
             }
         }
         return status_code;
@@ -593,7 +604,7 @@ int HttpHandler::defaultStaticHandler() {
             if (service->largeFileHandler) {
                 status_code = customHttpHandler(service->largeFileHandler);
             } else {
-                status_code = defaultLargeFileHandler();
+                status_code = defaultLargeFileHandler(filepath);
             }
         } else {
             status_code = HTTP_STATUS_NOT_FOUND;
@@ -618,10 +629,9 @@ int HttpHandler::defaultStaticHandler() {
     return status_code;
 }
 
-int HttpHandler::defaultLargeFileHandler() {
+int HttpHandler::defaultLargeFileHandler(const std::string &filepath) {
     if (!writer) return HTTP_STATUS_NOT_IMPLEMENTED;
     if (!isFileOpened()) {
-        std::string filepath = service->GetStaticFilepath(req->Path().c_str());
         if (filepath.empty() || openFile(filepath.c_str()) != 0) {
             return HTTP_STATUS_NOT_FOUND;
         }
@@ -817,7 +827,7 @@ return_header:
         {
             // NOTE: remove file cache if > FILE_CACHE_MAX_SIZE
             if (fc && fc->filebuf.len > FILE_CACHE_MAX_SIZE) {
-                files->Close(fc);
+                files->Close(fc->filepath.c_str());
             }
             fc = NULL;
             header.clear();
@@ -863,7 +873,11 @@ int HttpHandler::openFile(const char* filepath) {
     closeFile();
     file = new LargeFile;
     file->timer = INVALID_TIMER_ID;
+#ifdef OS_WIN
+    return file->open(hv::utf8_to_ansi(filepath).c_str(), "rb");
+#else
     return file->open(filepath, "rb");
+#endif
 }
 
 bool HttpHandler::isFileOpened() {
@@ -881,7 +895,7 @@ int HttpHandler::sendFile() {
     int readbytes = MIN(file->buf.len, resp->content_length);
     size_t nread = file->read(file->buf.base, readbytes);
     if (nread <= 0) {
-        hloge("read file error!");
+        hloge("read file: %s error!", file->filepath);
         error = ERR_READ_FILE;
         writer->close(true);
         return nread;
@@ -953,7 +967,7 @@ int HttpHandler::upgradeWebSocket() {
     if (iter_protocol != req->headers.end()) {
         hv::StringList subprotocols = hv::split(iter_protocol->second, ',');
         if (subprotocols.size() > 0) {
-            hlogw("%s: %s => just select first protocol %s", SEC_WEBSOCKET_PROTOCOL, iter_protocol->second.c_str(), subprotocols[0].c_str());
+            hlogw("[%s:%d] %s: %s => just select first protocol %s", ip, port, SEC_WEBSOCKET_PROTOCOL, iter_protocol->second.c_str(), subprotocols[0].c_str());
             resp->headers[SEC_WEBSOCKET_PROTOCOL] = subprotocols[0];
         }
     }
@@ -983,7 +997,7 @@ int HttpHandler::upgradeHTTP2() {
     SendHttpResponse();
 
     if (!SwitchHTTP2()) {
-        hloge("[%s:%d] unsupported HTTP2", ip, port);
+        hlogw("[%s:%d] unsupported HTTP2", ip, port);
         return SetError(ERR_INVALID_PROTOCOL);
     }
 
@@ -1010,7 +1024,7 @@ int HttpHandler::handleForwardProxy() {
     if (service && service->enable_forward_proxy) {
         return connectProxy(req->url);
     } else {
-        hlogw("Forbidden to forward proxy %s", req->url.c_str());
+        hlogw("[%s:%d] Forbidden to forward proxy %s", ip, port, req->url.c_str());
         SetError(HTTP_STATUS_FORBIDDEN, HTTP_STATUS_FORBIDDEN);
     }
     return 0;
@@ -1043,7 +1057,7 @@ int HttpHandler::connectProxy(const std::string& strUrl) {
     }
 
     if (forward_proxy && !service->IsTrustProxy(url.host.c_str())) {
-        hlogw("Forbidden to proxy %s", url.host.c_str());
+        hlogw("[%s:%d] Forbidden to proxy %s", ip, port, url.host.c_str());
         SetError(HTTP_STATUS_FORBIDDEN, HTTP_STATUS_FORBIDDEN);
         return 0;
     }
